////////////////////////////////////////////////////////////////////////
// epginfo.cpp
// generated with ecppc
//

#include <tnt/ecpp.h>
#include <tnt/convert.h>
#include <tnt/httprequest.h>
#include <tnt/httpreply.h>
#include <tnt/httpheader.h>
#include <tnt/http.h>
#include <tnt/data.h>
#include <tnt/componentfactory.h>
#include <cxxtools/log.h>
#include <stdexcept>

log_define("component.epginfo")

// <%pre>
#line 1 "epginfo.ecpp"

#include <sys/stat.h>
#include <vdr/tools.h>

#include "stdext.h"
#include "exception.h"
#include "setup.h"
#include "tools.h"
#include "epg_events.h"
#include "recman.h"

namespace vdrlive {
	class SchedulesLock
	{
		public:
			SchedulesLock() : m_schedulesLock() {}

			operator cSchedulesLock& () { return m_schedulesLock; }

		private:
			SchedulesLock(SchedulesLock const &schedulesLock) {}

			cSchedulesLock m_schedulesLock;
	};

	typedef std::tr1::shared_ptr<SchedulesLock> SchedulesLockPtr;
}

using namespace vdrlive;
using namespace std;


#line 1 "page_init.eh"

// do not add to Makefile
// and do not write a ecpp comment into this file. It must produce no
// html output not even a empty line.
#include <tnt/savepoint.h>
#include "exception.h"

#line 2 "page_exit.eh"

#include "exception.h"

// </%pre>

namespace
{
class _component_ : public tnt::EcppComponent
{
    _component_& main()  { return *this; }

  protected:
    ~_component_();

  public:
    _component_(const tnt::Compident& ci, const tnt::Urlmapper& um, tnt::Comploader& cl);

    unsigned operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam);
    class epgTools_type : public tnt::EcppSubComponent
    {
        _component_& mainComp;
        _component_& main()  { return mainComp; }

      public:
        epgTools_type(_component_& m, const std::string& name)
          : EcppSubComponent(m, name),
            mainComp(m)
          { }
        unsigned operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam);
    };

    epgTools_type epgTools;
};

static tnt::ComponentFactoryImpl<_component_> Factory("epginfo");

static const char* rawData = "\000\000\000P\000\000\000t\000\000\000}\000\000\000\177\000\000\000\202\000\000\000\203"
  "\000\000\000\224\000\000\000\226\000\000\000\231\000\000\000\232\000\000\000\261\000\000\000\263\000\000\000\264\000"
  "\000\000\266\000\000\000\267\000\000\000\322\000\000\000\323\000\000\000\324\000\000\000\326\000\000\000\327\n<html>"
  "\n\t<head>\n\t\t<title>VDR-Live - </title>\n\t\t\n\t\t\n\t</head>\n\t<body>\n\t\t\n\t\t\n\t\t<div class=\"inhalt\">\n"
  "\t\t\n\t\t\n\t\t</div>\n\t</body>\n</html>\n\n\n\n\n\n\n";

// <%shared>
// </%shared>

// <%config>
// </%config>

#define SET_LANG(lang) \
     do \
     { \
       request.setLang(lang); \
       reply.setLocale(request.getLocale()); \
     } while (false)

_component_::_component_(const tnt::Compident& ci, const tnt::Urlmapper& um, tnt::Comploader& cl)
  : EcppComponent(ci, um, cl),
    epgTools(*this, "epgTools")
{
  // <%init>
  // </%init>
}

_component_::~_component_()
{
  // <%cleanup>
  // </%cleanup>
}

unsigned _component_::operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam)
{
  log_trace("epginfo " << qparam.getUrl());

  tnt::DataChunks data(rawData);

  // <%args>
string epgid = tnt::stringTo<string>("epgid", "string", qparam.param("epgid"), reply.out().getloc());
string async = tnt::stringTo<string>("async", "string", qparam.param("async"), reply.out().getloc());
  // </%args>

#line 38 "epginfo.ecpp"
  typedef bool logged_in_type;
  TNT_SESSION_GLOBAL_VAR(logged_in_type, logged_in, "bool logged_in", (false));   // <%session> bool logged_in(false)
#line 41 "epginfo.ecpp"
  typedef EpgInfoPtr epgEvent_type;
  TNT_REQUEST_PAGE_VAR(epgEvent_type, epgEvent, "EpgInfoPtr epgEvent", ());   // <%request> EpgInfoPtr epgEvent
#line 9 "page_init.eh"
  typedef std::string pageTitle_type;
  TNT_REQUEST_GLOBAL_VAR(pageTitle_type, pageTitle, "std::string pageTitle", ());   // <%request> std::string pageTitle
  // <%cpp>
  // <%include> page_init.eh
#line 11 "page_init.eh"

try {
	reply.setHeader("Expires", "Mon, 26 Jul 1997 05:00:00 GMT");
	tnt::Savepoint spoint( reply );

  // </%include>
#line 44 "epginfo.ecpp"

	if (!logged_in && LiveSetup().UseAuth()) return reply.redirect("login.html");

	pageTitle = tr("Electronic program guide information");

	bool ajaxReq = !async.empty() && (lexical_cast<int>(async) != 0);

	bool aboutBox = false;

	// One of these get initialized when needed. When freed by getting
	// out of scope they will release (if initialized) important
	// Semaphores/Locks.
	SchedulesLockPtr schedulesLockPtr;
	RecordingsManagerPtr recordings;

	if (!epgid.empty()) {

		const string recording("recording_");
		const string event("event_");
		const string aboutbox("aboutBox");

		// check for recording:
		if (epgid.compare(0, recording.length(), recording) == 0) {
			recordings = LiveRecordingsManager();
			const cRecording* recording = recordings->GetByMd5Hash(epgid);
			if (recording == 0) {
				throw HtmlError(tr("Couldn't find recording or no recordings available"));
			}
			epgEvent = EpgEvents::CreateEpgInfo(epgid, recording);
		}
		// check for event:
		else if (epgid.compare(0, event.length(), event) == 0) {
			schedulesLockPtr = SchedulesLockPtr(new SchedulesLock);
			if (!schedulesLockPtr) {
				throw HtmlError(tr("Error aquiring schedules lock"));
			}
			const cSchedules* schedules = cSchedules::Schedules(*schedulesLockPtr);
			if (!schedules) {
				throw HtmlError(tr("Error aquiring schedules"));
			}
			epgEvent = EpgEvents::CreateEpgInfo(epgid, schedules);
		}
		// check for aboutbox:
		else if (epgid.compare(0, aboutbox.length(), aboutbox) == 0) {
			aboutBox = true;
		}
	}

  // <& pageelems.doc_type ...
  { 
#line 92 "epginfo.ecpp"
    tnt::QueryParams _tnt_cq0;
#line 92 "epginfo.ecpp"
    callComp(tnt::Subcompident(std::string(), "pageelems", "doc_type"), request, reply, _tnt_cq0);
    // &>
  }
  reply.out() << data[0]; // \n<html>\n\t<head>\n\t\t<title>VDR-Live - 
#line 95 "epginfo.ecpp"
  reply.sout() << ( pageTitle );
  reply.out() << data[1]; // </title>\n
#line 96 "epginfo.ecpp"

	if (!ajaxReq) {

  reply.out() << data[2]; // \t\t
  // <& pageelems.stylesheets ...
  { 
#line 99 "epginfo.ecpp"
    tnt::QueryParams _tnt_cq1;
#line 99 "epginfo.ecpp"
    callComp(tnt::Subcompident(std::string(), "pageelems", "stylesheets"), request, reply, _tnt_cq1);
    // &>
  }
  reply.out() << data[3]; // \n\t\t
  // <& pageelems.ajax_js ...
  { 
#line 100 "epginfo.ecpp"
    tnt::QueryParams _tnt_cq2;
#line 100 "epginfo.ecpp"
    callComp(tnt::Subcompident(std::string(), "pageelems", "ajax_js"), request, reply, _tnt_cq2);
    // &>
  }
  reply.out() << data[4]; // \n
#line 101 "epginfo.ecpp"

	}

  reply.out() << data[5]; // \t</head>\n\t<body>\n
#line 106 "epginfo.ecpp"

	if (!ajaxReq) {

  reply.out() << data[6]; // \t\t
  // <& pageelems.logo ...
  { 
#line 109 "epginfo.ecpp"
    tnt::QueryParams _tnt_cq3;
#line 109 "epginfo.ecpp"
    callComp(tnt::Subcompident(std::string(), "pageelems", "logo"), request, reply, _tnt_cq3);
    // &>
  }
  reply.out() << data[7]; // \n\t\t
  // <& menu ...
  { 
#line 110 "epginfo.ecpp"
    tnt::QueryParams _tnt_cq4;
#line 110 "epginfo.ecpp"
    callComp(tnt::Compident(std::string(), "menu"), request, reply, _tnt_cq4);
    // &>
  }
  reply.out() << data[8]; // \n
#line 111 "epginfo.ecpp"

	}

  reply.out() << data[9]; // \t\t<div class="inhalt">\n
#line 115 "epginfo.ecpp"

	if (epgEvent) {
		string start(epgEvent->StartTime("%a,") + string(" ")
				+ epgEvent->StartTime(tr("%b %d %y")) + string(" ")
				+ epgEvent->StartTime(tr("%I:%M %p")));
		string tools_component;
		if (recordings) {
			tools_component = epgEvent->Archived().empty() ? "recordings.rec_tools" : "recordings.archived_disc" ;
		}
		if (schedulesLockPtr) {
			tools_component = "epginfo.epgTools";
		}

  reply.out() << data[10]; // \t\t
  // <& pageelems.epg_tt_box ...
  { 
#line 128 "epginfo.ecpp"
    tnt::QueryParams _tnt_cq5;
#line 128 "epginfo.ecpp"
    _tnt_cq5.add( "archived", tnt::toString((epgEvent->Archived()), reply.out().getloc() ) );
#line 128 "epginfo.ecpp"
    _tnt_cq5.add( "boxId", tnt::toString((epgEvent->Id()), reply.out().getloc() ) );
#line 128 "epginfo.ecpp"
    _tnt_cq5.add( "caption", tnt::toString((epgEvent->Caption()), reply.out().getloc() ) );
#line 128 "epginfo.ecpp"
    _tnt_cq5.add( "elapsed", tnt::toString((epgEvent->Elapsed()), reply.out().getloc() ) );
#line 128 "epginfo.ecpp"
    _tnt_cq5.add( "long_descr", tnt::toString((epgEvent->LongDescr()), reply.out().getloc() ) );
#line 128 "epginfo.ecpp"
    _tnt_cq5.add( "short_descr", tnt::toString((epgEvent->ShortDescr()), reply.out().getloc() ) );
#line 128 "epginfo.ecpp"
    _tnt_cq5.add( "time", tnt::toString((start), reply.out().getloc() ) );
#line 128 "epginfo.ecpp"
    _tnt_cq5.add( "title", tnt::toString((epgEvent->Title()), reply.out().getloc() ) );
#line 128 "epginfo.ecpp"
    _tnt_cq5.add( "tools_comp", tnt::toString((tools_component), reply.out().getloc() ) );
#line 128 "epginfo.ecpp"
    callComp(tnt::Subcompident(std::string(), "pageelems", "epg_tt_box"), request, reply, _tnt_cq5);
    // &>
  }
  reply.out() << data[11]; // \n
#line 129 "epginfo.ecpp"

	}
	if (aboutBox) {

  reply.out() << data[12]; // \t\t
  // <& pageelems.about_tt_box ...
  { 
#line 133 "epginfo.ecpp"
    tnt::QueryParams _tnt_cq6;
#line 133 "epginfo.ecpp"
    callComp(tnt::Subcompident(std::string(), "pageelems", "about_tt_box"), request, reply, _tnt_cq6);
    // &>
  }
  reply.out() << data[13]; // \n
#line 134 "epginfo.ecpp"

	}

  reply.out() << data[14]; // \t\t</div>\n\t</body>\n</html>\n\n
  // <%include> page_exit.eh
  reply.out() << data[15]; // \n
#line 5 "page_exit.eh"

	spoint.commit();
} catch ( vdrlive::HtmlError const& ex ) {
	tnt::QueryParams param = qparam;
	param.add( "pageTitle", pageTitle );
	param.add( "errorMessage", ex.what() );
	callComp( "error", request, reply, param );
}

  // </%include>
  reply.out() << data[16]; // \n
  reply.out() << data[17]; // \n\n
  // <%/cpp>
  return HTTP_OK;
}

unsigned _component_::epgTools_type::operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam)
{
  log_trace("epginfo::epgTools " << qparam.getUrl());
  tnt::DataChunks data(rawData);
  // <%args>
string id = tnt::stringTo<string>("id", "string", qparam.param("id"), reply.out().getloc());
string title = tnt::stringTo<string>("title", "string", qparam.param("title"), reply.out().getloc());
int detail = tnt::stringTo<int>("detail", "int", qparam.param("detail"), reply.out().getloc());
  // </%args>

#line 41 "epginfo.ecpp"
  typedef EpgInfoPtr epgEvent_type;
  TNT_REQUEST_PAGE_VAR(epgEvent_type, epgEvent, "EpgInfoPtr epgEvent", ());   // <%request> EpgInfoPtr epgEvent
#line 38 "epginfo.ecpp"
  typedef bool logged_in_type;
  TNT_SESSION_GLOBAL_VAR(logged_in_type, logged_in, "bool logged_in", (false));   // <%session> bool logged_in(false)
#line 9 "page_init.eh"
  typedef std::string pageTitle_type;
  TNT_REQUEST_GLOBAL_VAR(pageTitle_type, pageTitle, "std::string pageTitle", ());   // <%request> std::string pageTitle
  // <%cpp>
  // <& pageelems.epg_tool_box ...
  { 
#line 151 "epginfo.ecpp"
    tnt::QueryParams _tnt_cq7;
#line 151 "epginfo.ecpp"
    _tnt_cq7.add( "detail", tnt::toString((detail), reply.out().getloc() ) );
#line 151 "epginfo.ecpp"
    _tnt_cq7.add( "endTime", tnt::toString((epgEvent->GetEndTime()), reply.out().getloc() ) );
#line 151 "epginfo.ecpp"
    _tnt_cq7.add( "epgid", tnt::toString((id), reply.out().getloc() ) );
#line 151 "epginfo.ecpp"
    _tnt_cq7.add( "startTime", tnt::toString((epgEvent->GetStartTime()), reply.out().getloc() ) );
#line 151 "epginfo.ecpp"
    _tnt_cq7.add( "title", tnt::toString((title), reply.out().getloc() ) );
#line 151 "epginfo.ecpp"
    callComp(tnt::Subcompident(std::string(), "pageelems", "epg_tool_box"), request, reply, _tnt_cq7);
    // &>
  }
  reply.out() << data[18]; // \n
  // <%/cpp>
  return HTTP_OK;
}

} // namespace
