////////////////////////////////////////////////////////////////////////
// recstream.cpp
// generated with ecppc
//

#include <tnt/ecpp.h>
#include <tnt/convert.h>
#include <tnt/httprequest.h>
#include <tnt/httpreply.h>
#include <tnt/httpheader.h>
#include <tnt/http.h>
#include <tnt/data.h>
#include <tnt/componentfactory.h>
#include <cxxtools/log.h>
#include <stdexcept>

log_define("component.recstream")

// <%pre>
#line 1 "recstream.ecpp"

#include <string>
#include <iostream>
#include <fstream>
#include <algorithm>
#include <tnt/httperror.h>
#include <tnt/httpheader.h>
#include <vdr/recording.h>
#include "tntfeatures.h"
#include "setup.h"
#include "recman.h"

using namespace std;
using namespace vdrlive;

off_t RecSize(cRecording const * recording)
{
    cFileName recFile(recording->FileName(), false, false, recording->IsPesRecording());
    off_t recSize = 0;
    for (cUnbufferedFile *recData = recFile.Open(); recData; recData = recFile.NextFile()) {
        struct stat buf;
        if (0 == stat(recFile.Name(), &buf)) {
            recSize += buf.st_size;
            // dsyslog("LIVE: size of recording part %s is %ld", recFile.Name(), buf.st_size);
        }
        else {
            esyslog("LIVE: can't determine size of %s", recFile.Name());
        }
    }
    // dsyslog("LIVE: total size of %s is %ld", recording->FileName(), recSize);
    return recSize;
}


// </%pre>

namespace
{
class _component_ : public tnt::EcppComponent
{
    _component_& main()  { return *this; }

  protected:
    ~_component_();

  public:
    _component_(const tnt::Compident& ci, const tnt::Urlmapper& um, tnt::Comploader& cl);

    unsigned operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam);
};

static tnt::ComponentFactoryImpl<_component_> Factory("recstream");

// <%shared>
// </%shared>

// <%config>
// </%config>

#define SET_LANG(lang) \
     do \
     { \
       request.setLang(lang); \
       reply.setLocale(request.getLocale()); \
     } while (false)

_component_::_component_(const tnt::Compident& ci, const tnt::Urlmapper& um, tnt::Comploader& cl)
  : EcppComponent(ci, um, cl)
{
  // <%init>
  // </%init>
}

_component_::~_component_()
{
  // <%cleanup>
  // </%cleanup>
}

unsigned _component_::operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam)
{
  log_trace("recstream " << qparam.getUrl());


  // <%args>
string recid = tnt::stringTo<string>("recid", "string", qparam.param("recid"), reply.out().getloc());
  // </%args>

#line 39 "recstream.ecpp"
  typedef bool logged_in_type;
  TNT_SESSION_GLOBAL_VAR(logged_in_type, logged_in, "bool logged_in", (false));   // <%session> bool logged_in(false)
  // <%cpp>
#line 41 "recstream.ecpp"

//if (!logged_in && LiveSetup().UseAuth()) return reply.redirect("login.html");

cRecording const * recording = LiveRecordingsManager()->GetByMd5Hash(recid);
if (recording) {
	reply.setContentType("video/mpeg");
    reply.setKeepAliveHeader();
    reply.setContentLengthHeader(RecSize(recording));
    reply.setDirectMode();

    cFileName recFile(recording->FileName(), false, false, recording->IsPesRecording());
    // dsyslog("LIVE: start send video data.");
    for (cUnbufferedFile *recData = recFile.Open(); recData; recData = recFile.NextFile()) {
        char buffer[KILOBYTE(16)];
        ssize_t bytesRead = 0;
        // dsyslog("LIVE: send file %s", recFile->Name());
        while (0 < (bytesRead = recData->Read(buffer, sizeof(buffer)))) {
            // dsyslog("LIVE: copy %zd bytes", bytesRead);
            reply.out().write(buffer, bytesRead);
            if (!reply.out()) {
                return HTTP_GONE;
            }
#if TNT_WATCHDOG_SILENCE
            request.touch(); // retrigger the watchdog.
#endif
        }
        // dsyslog("LIVE: bytesRead = %zd", bytesRead);
        if (bytesRead < 0) {
            return HTTP_PARTIAL_CONTENT;
        }
    }
    // dsyslog("LIVE: finished send video data.");
    reply.out() << std::flush;
    return HTTP_OK;
}
return DECLINED;

  // <%/cpp>
  return HTTP_OK;
}

} // namespace
